#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAXLEN 127

#define DELIM " \n\t"

double
math(char op, double a, double b)
{
    switch (op) {
    case '+':
	return (a+b);
    case '-':
	return (a-b);
    case '*':
	return (a*b);
    case '/':
	if (b == 0.0) {
	    fputs("!DIV by zero\n", stderr);
	    return 0.0;
	}
	return (a/b);
    }

    /* we should never get here */

    fputs("!Unrecognized operation\n", stderr);
    return 0.0;
}

int
main(int argc, char **argv)
{
    char *inp;
    double a, b, result = 0.0;
    char op;
    char *token;
    int count;

    if (argc != 1) {
	fputs("!Too many arguments\n", stderr);
	fputs("Usage: rc\n", stderr);
    }

    /* allocate memory */

    inp = calloc(MAXLEN + 1, sizeof(char));

    if (inp == NULL) {
	puts("!Out of memory");
	return 1;
    }

    /* print a prompt */

    fputs("RC - Running Calculator\n", stderr);
    fputs("Syntax: op num [num]\n", stderr);
    fputs("Operators: = + - * / (q=quit)\n", stderr);

    /* read input and perform calc */

    do {
	/* print the prompt */

	printf("%lf\n", result);
	putchar(':');

	/* read the input */

	fgets(inp, MAXLEN, stdin);

	inp[MAXLEN] = 0; /* guarantee null-term */

	/* puts(inp); /* debugging */

	/* clear vars */

	op = 0;
	a = 0.0;
	b = 0.0;

	count = 0;

	/* the first token */

	token = strtok(inp, DELIM);

	if (token != NULL) {
	    if (strlen(token) > 1) {
		fputs("!Operator is too long\n", stderr);
	    }
	    else {
		op = token[0];
		count++;

		/* the second token */

		token = strtok(NULL, DELIM);

		if (token != NULL) {
		    a = atof(token);
		    count++;

		    /* the third token */

		    token = strtok(NULL, DELIM);

		    if (token != NULL) {
			b = atof(token);
			count++;

			/* this next call should return NULL */

			token = strtok(NULL, DELIM);

			if (token != NULL) {
			    fputs("!Extra options ignored\n", stderr);
			}
		    }
		}
	    }
	}

	if ( (op == '+') || (op == '-') || (op == '*') || (op == '/')) {
	    /* do some math */

	    switch (count) {
	    case 3: /* op a b */
		result = math(op,a,b);
		break;

	    case 2: /* op a */
		result = math(op,result,a);
		break;

	    case 1: /* op */
		result = math(op,result,0.0);
		break;

	    default:
		fputs("!Overflow\n", stderr); /* we should never see this */
		break;

	    case 0: /* empty - ignored */
		break;
	    }
	}
	else if (op == '=') {
	    /* set a value */
	    result = a;
	}
	else if ( (op != 'q') && (op != 'Q') ) {
	    /* ?? */
	    fputs("!Unrecognized operator\n", stderr);
	}

    } while ((op != 'q') && (op != 'Q'));

    /* done */

    free(inp);

    return 0;
}
